@@ -16,6 +16,7 @@ from ..configs import Configs
 from ..misc.toolkit import Initializer
 from ..models.base import model_dict
 from ..models.base import ModelBase
+from ..models.base import SiameseModelBase
 from ..modules.heads import head_dict
 from ..modules.heads import HeadBase
 from ..modules.heads import HeadConfigs
@@ -56,6 +57,7 @@ class PipeInfo(NamedTuple):
     key: str
     transform: str = "default"
     extractor: Optional[str] = None
+    reuse_extractor: bool = True
     head: Optional[str] = None
     extractor_config: str = "default"
     head_config: str = "default"
@@ -68,6 +70,7 @@ def register_pipe(
     *,
     transform: str = "default",
     extractor: Optional[str] = None,
+    reuse_extractor: bool = True,
     head: Optional[str] = None,
     extractor_config: str = "default",
     head_config: str = "default",
@@ -80,6 +83,7 @@ def register_pipe(
         extractor=extractor,
         head=head,
         extractor_config=extractor_config,
+        reuse_extractor=reuse_extractor,
         head_config=head_config,
         extractor_meta_scope=extractor_meta_scope,
         head_meta_scope=head_meta_scope,
@@ -99,16 +103,25 @@ def register_model(
         pass
 
     for pipe in pipes:
-        _ = register_pipe(  # type: ignore
-            pipe.key,
-            transform=pipe.transform,
-            extractor=pipe.extractor,
-            head=pipe.head,
-            extractor_config=pipe.extractor_config,
-            head_config=pipe.head_config,
-            extractor_meta_scope=pipe.extractor_meta_scope,
-            head_meta_scope=pipe.head_meta_scope,
-        )(_)
+        _ = register_pipe(**pipe._asdict())(_)  # type: ignore
+
+    return None
+
+
+def register_siamese_model(
+    name: str,
+    *,
+    pipes: Optional[List[PipeInfo]] = None,
+) -> Optional[Callable[[Type], Type]]:
+    if pipes is None:
+        return SiameseModelBase.register(name)
+
+    @SiameseModelBase.register(name)
+    class _(SiameseModelBase):
+        pass
+
+    for pipe in pipes:
+        _ = register_pipe(**pipe._asdict())(_)  # type: ignore
 
     return None
 
@@ -252,6 +265,7 @@ __all__ = [
     "register_aggregator",
     "register_pipe",
     "register_model",
+    "register_siamese_model",
     "register_config",
     "register_head_config",
     "register_metric",
