@@ -7,6 +7,7 @@ import platform
 
 import numpy as np
 import torch.nn as nn
+import torch.nn.functional as F
 
 from typing import *
 from abc import abstractmethod
@@ -63,7 +64,7 @@ def to_2d(arr: data_type) -> data_type:
 
 
 def to_prob(raw: np.ndarray) -> np.ndarray:
-    return nn.functional.softmax(torch.from_numpy(raw), dim=1).numpy()
+    return F.softmax(torch.from_numpy(raw), dim=1).numpy()
 
 
 def collate_np_dicts(ds: List[np_dict_type], axis: int = 0) -> np_dict_type:
@@ -194,7 +195,12 @@ def inject_mlflow_stuffs(
 
 # This is a modified version of https://github.com/sksq96/pytorch-summary
 #  So it can summary `carefree-learn` model structures better
-def summary(model: nn.Module, sample_batch: tensor_dict_type) -> None:
+def summary(
+    model: nn.Module,
+    sample_batch: tensor_dict_type,
+    *,
+    return_only: bool = False,
+) -> str:
     def register_hook(module: nn.Module) -> None:
         def hook(module_: nn.Module, inp: Any, output: Any) -> None:
             m_name = module_names.get(module_)
@@ -331,12 +337,12 @@ def summary(model: nn.Module, sample_batch: tensor_dict_type) -> None:
     _inject_summary(hierarchy, [])
 
     line_length = 120
-    print("=" * line_length)
+    messages = ["=" * line_length]
     line_format = "{:30}  {:>20} {:>40} {:>20}"
     headers = "Layer (type)", "Input Shape", "Output Shape", "Trainable Param #"
     line_new = line_format.format(*headers)
-    print(line_new)
-    print("-" * line_length)
+    messages.append(line_new)
+    messages.append("-" * line_length)
     total_output = 0
     for layer in summary_dict:
         # name, input_shape, output_shape, num_trainable_params
@@ -352,7 +358,7 @@ def summary(model: nn.Module, sample_batch: tensor_dict_type) -> None:
             output_shape = [output_shape]
         for shape in output_shape:
             total_output += prod(shape)
-        print(line_new)
+        messages.append(line_new)
 
     # assume 4 bytes/number (float on cuda).
     x_batch = sample_batch["x_batch"]
@@ -362,16 +368,21 @@ def summary(model: nn.Module, sample_batch: tensor_dict_type) -> None:
     total_params_size = abs(total_params * 4.0 / (1024 ** 2.0))
     total_size = total_params_size + total_output_size + total_input_size
 
-    print("=" * line_length)
-    print("Total params: {0:,}".format(total_params))
-    print("Trainable params: {0:,}".format(trainable_params))
-    print("Non-trainable params: {0:,}".format(total_params - trainable_params))
-    print("-" * line_length)
-    print("Input size (MB): %0.2f" % total_input_size)
-    print("Forward/backward pass size (MB): %0.2f" % total_output_size)
-    print("Params size (MB): %0.2f" % total_params_size)
-    print("Estimated Total Size (MB): %0.2f" % total_size)
-    print("-" * line_length)
+    non_trainable_params = total_params - trainable_params
+    messages.append("=" * line_length)
+    messages.append("Total params: {0:,}".format(total_params))
+    messages.append("Trainable params: {0:,}".format(trainable_params))
+    messages.append("Non-trainable params: {0:,}".format(non_trainable_params))
+    messages.append("-" * line_length)
+    messages.append("Input size (MB): %0.2f" % total_input_size)
+    messages.append("Forward/backward pass size (MB): %0.2f" % total_output_size)
+    messages.append("Params size (MB): %0.2f" % total_params_size)
+    messages.append("Estimated Total Size (MB): %0.2f" % total_size)
+    messages.append("-" * line_length)
+    msg = "\n".join(messages)
+    if not return_only:
+        print(msg)
+    return msg
 
 
 class LoggingMixinWithRank(LoggingMixin):
@@ -576,7 +587,7 @@ class Activations:
         try:
             return getattr(nn, item)(**kwargs)
         except AttributeError:
-            func = getattr(torch, item, getattr(nn.functional, item, None))
+            func = getattr(torch, item, getattr(F, item, None))
             if func is None:
                 raise NotImplementedError(
                     "neither pytorch nor custom Activations "
@@ -614,7 +625,7 @@ class Activations:
     def mish(self) -> nn.Module:
         class Mish(nn.Module):
             def forward(self, x: torch.Tensor) -> torch.Tensor:
-                return x * (torch.tanh(nn.functional.softplus(x)))
+                return x * (torch.tanh(F.softplus(x)))
 
         return Mish()
 
@@ -709,7 +720,7 @@ class Activations:
                 self.dim = dim
 
             def _core(self, multiplied: torch.Tensor) -> torch.Tensor:
-                return nn.functional.softmax(multiplied, dim=self.dim)
+                return F.softmax(multiplied, dim=self.dim)
 
         return MultipliedSoftmax(**self.configs.setdefault("multiplied_softmax", {}))
 
@@ -734,7 +745,7 @@ class Activations:
 
             def forward(self, net: torch.Tensor) -> torch.Tensor:
                 net_abs = net.abs()
-                bias = nn.functional.softplus(self.bias)
+                bias = F.softplus(self.bias)
                 cup_mask = self.sigmoid(net_abs - bias)
                 masked = net_abs * cup_mask
                 if not self.retain_sign:
