@@ -2,7 +2,7 @@ from .base import ModelBase
 
 
 @ModelBase.register("transformer")
-@ModelBase.register_pipe("transformer", head="linear")
+@ModelBase.register_pipe("transformer", head="fcnn", head_config="highway")
 class Transformer(ModelBase):
     pass
 
