@@ -1154,17 +1154,15 @@ class Attention(Module):
         self.input_dim = input_dim
         self.is_self_attn = is_self_attention
         if not is_self_attention:
-            self.k_dim = k_dim if k_dim is not None else input_dim
-            self.v_dim = v_dim if v_dim is not None else input_dim
+            self.k_dim = k_dim or input_dim
+            self.v_dim = v_dim or input_dim
         else:
             if k_dim is not None and k_dim != input_dim:
                 raise ValueError("self attention is used but `k_dim` != `input_dim`")
             if v_dim is not None and v_dim != input_dim:
                 raise ValueError("self attention is used but `v_dim` != `input_dim`")
             self.k_dim = self.v_dim = input_dim
-        if embed_dim is None:
-            embed_dim = min(32, input_dim) * num_heads
-        self.embed_dim = embed_dim
+        self.embed_dim = embed_dim or input_dim
 
         self.num_heads = num_heads
         self.head_dim = self.embed_dim // num_heads
@@ -1251,7 +1249,7 @@ class Attention(Module):
         raw_weights = torch.bmm(q, k.transpose(-2, -1))
         if mask is not None:
             raw_weights.masked_fill_(mask, float("-inf"))
-        # B * N_head, Sq, Sk -> # B * N_head, Sq, Sk
+        # B * N_head, Sq, Sk -> B * N_head, Sq, Sk
         weights = self._get_weights(raw_weights)
         if 0.0 < self.dropout < 1.0:
             weights = F.dropout(weights, self.dropout, self.training)
