@@ -1,5 +1,6 @@
 from .core import Identity
+from .core import IdentityTS
 from .configs import DefaultIdentityConfig
 
 
-__all__ = ["Identity"]
+__all__ = ["Identity", "IdentityTS"]
