@@ -11,13 +11,15 @@ class DefaultTransformerConfig(Configs):
             "num_heads": 4,
             "num_layers": 3,
             "latent_dim": 32,
-            "dropout": 0.0,
+            "dropout": 0.1,
             "norm_type": "layer_norm",
             "attention_type": "decayed",
             "encoder_type": "basic",
-            "input_linear_config": {},
+            "input_linear_config": {"bias": False},
             "layer_config": {"latent_dim": 128},
             "encoder_config": {},
+            "use_head_token": True,
+            "final_norm_type": "layer_norm",
         }
 
 
