@@ -4,7 +4,6 @@ import torch
 
 import numpy as np
 import torch.nn as nn
-import torch.nn.functional as F
 
 from typing import *
 from torch import Tensor
@@ -34,6 +33,61 @@ def _get_norm(norm_type: str, dim: int) -> nn.Module:
     return base(dim)
 
 
+class Residual(nn.Module):
+    def __init__(self, module: nn.Module):
+        super().__init__()
+        self.module = module
+
+    def forward(self, x: Tensor, **kwargs: Any) -> Tensor:
+        return self.module(x, **kwargs)
+
+
+class PreNorm(nn.Module):
+    def __init__(self, *dims: int, module: nn.Module, norm_type: str = "layer_norm"):
+        super().__init__()
+        self.norms = nn.ModuleList([])
+        for dim in dims:
+            self.norms.append(_get_norm(norm_type, dim))
+        self.module = module
+
+    def forward(self, *xs: Tensor, **kwargs: Any) -> Tensor:
+        x_list = [norm(x) for x, norm in zip(xs, self.norms)]
+        if not issubclass(self.module.__class__, Attention):
+            return self.module(*x_list, **kwargs)
+        if len(x_list) == 1:
+            x_list = [x_list[0]] * 3
+        elif len(x_list) == 2:
+            x_list.append(x_list[1])
+        if len(x_list) != 3:
+            raise ValueError("there should be three inputs for `Attention`")
+        return self.module(*x_list, **kwargs).output
+
+
+class FeedForward(nn.Module):
+    def __init__(
+        self,
+        in_dim: int,
+        latent_dim: int,
+        *,
+        dropout: float = 0.0,
+        activation: str = "GELU",
+        activation_config: Optional[Dict[str, Any]] = None,
+        to_latent_config: Optional[Dict[str, Any]] = None,
+        from_latent_config: Optional[Dict[str, Any]] = None,
+    ):
+        super().__init__()
+        self.net = nn.Sequential(
+            Linear(in_dim, latent_dim, **(to_latent_config or {})),
+            Activations.make(activation, activation_config),
+            Dropout(dropout),
+            Linear(latent_dim, in_dim, **(from_latent_config or {})),
+            Dropout(dropout),
+        )
+
+    def forward(self, x: Tensor) -> Tensor:
+        return self.net(x)
+
+
 @Attention.register("decayed")
 class DecayedAttention(Attention):
     def __init__(
@@ -74,7 +128,6 @@ class DecayedAttention(Attention):
         mask = np.zeros([seq_len, seq_len], dtype=np.float32)
         for i in range(1, seq_len):
             np.fill_diagonal(mask[i:], i ** 2)
-            np.fill_diagonal(mask[..., i:], i ** 2)
         mask_ = torch.from_numpy(mask)
         decayed_mask = torch.empty(num_heads, seq_len, seq_len)
         for i in range(num_heads):
@@ -97,8 +150,8 @@ class TransformerLayer(nn.Module):
         *,
         dropout: float = 0.1,
         latent_dim: int = 2048,
-        activation: str = "ReLU",
-        norm_type: str = "batch_norm",
+        activation: str = "GELU",
+        norm_type: str = "layer_norm",
         attention_type: str = "decayed",
         seq_len: Optional[int] = None,
         attention_config: Optional[Dict[str, Any]] = None,
@@ -107,6 +160,7 @@ class TransformerLayer(nn.Module):
         from_latent_config: Optional[Dict[str, Any]] = None,
     ):
         super().__init__()
+        # attention
         if attention_config is None:
             attention_config = {}
         attention_config["is_self_attention"] = True
@@ -117,28 +171,33 @@ class TransformerLayer(nn.Module):
                 msg = "`seq_len` should be provided when `decayed` attention is used"
                 raise ValueError(msg)
             attention_config["seq_len"] = seq_len
-        self.self_attn = attn_base(input_dim, num_heads, **attention_config)
-        if to_latent_config is None:
-            to_latent_config = {}
-        self.to_latent = Linear(input_dim, latent_dim, **to_latent_config)
-        self.dropout = Dropout(dropout)
-        if from_latent_config is None:
-            from_latent_config = {}
-        self.from_latent = Linear(latent_dim, input_dim, **from_latent_config)
-
-        self.norm1 = _get_norm(norm_type, input_dim)
-        self.norm2 = _get_norm(norm_type, input_dim)
-        self.dropout1 = Dropout(dropout)
-        self.dropout2 = Dropout(dropout)
-        self.activation = Activations.make(activation, activation_config)
+        self.attention_block = Residual(
+            PreNorm(
+                input_dim,
+                module=attn_base(input_dim, num_heads, **attention_config),
+                norm_type=norm_type,
+            )
+        )
+        # feed forward
+        self.feed_forward_block = Residual(
+            PreNorm(
+                input_dim,
+                module=FeedForward(
+                    input_dim,
+                    latent_dim,
+                    dropout=dropout,
+                    activation=activation,
+                    activation_config=activation_config,
+                    to_latent_config=to_latent_config,
+                    from_latent_config=from_latent_config,
+                ),
+                norm_type=norm_type,
+            )
+        )
 
     def forward(self, net: Tensor, mask: Optional[Tensor] = None) -> Tensor:
-        new = self.self_attn(net, net, net, mask=mask).output
-        net = net + self.dropout1(new)
-        net = self.norm1(net)
-        new = self.from_latent(self.dropout(self.activation(self.to_latent(net))))
-        net = net + self.dropout2(new)
-        net = self.norm2(net)
+        net = self.attention_block(net, mask=mask)
+        net = self.feed_forward_block(net)
         return net
 
 
@@ -151,10 +210,9 @@ class TransformerEncoder(nn.Module):
         encoder_layer: nn.Module,
         num_layers: int,
         dimensions: Dimensions,
-        norm: Optional[nn.Module] = None,
+        **kwargs: Any,
     ):
         super().__init__()
-        self.norm = norm
         self.dimensions = dimensions
         self.layers = _get_clones(encoder_layer, num_layers)
 
@@ -169,8 +227,6 @@ class TransformerEncoder(nn.Module):
     def forward(self, net: Tensor, mask: Optional[Tensor]) -> Tensor:
         for i, layer in enumerate(self.layers):
             net = layer(net, mask=self._get_mask(i, net, mask))
-        if self.norm:
-            net = self.norm(net)
         return net
 
     @classmethod
@@ -223,13 +279,20 @@ class Transformer(ExtractorBase):
         input_linear_config: Dict[str, Any],
         layer_config: Dict[str, Any],
         encoder_config: Dict[str, Any],
+        use_head_token: bool,
+        final_norm_type: Optional[str],
     ):
         super().__init__(in_flat_dim, dimensions)
         seq_len = dimensions.num_history
         # latent projection
-        self.scaling = float(latent_dim) ** 0.5
         self.latent_dim = latent_dim
         self.input_linear = Linear(self.in_dim, latent_dim, **input_linear_config)
+        # head token
+        if not use_head_token:
+            self.head_token = None
+        else:
+            seq_len += 1
+            self.head_token = nn.Parameter(torch.randn(1, 1, latent_dim))
         # position encoding
         self.position_encoding = PositionalEncoding(latent_dim, seq_len, dropout)
         # transformer blocks
@@ -241,7 +304,10 @@ class Transformer(ExtractorBase):
         layer = TransformerLayer(latent_dim, num_heads, **layer_config)
         encoder_base = TransformerEncoder.get(encoder_type)
         self.encoder = encoder_base(layer, num_layers, dimensions, **encoder_config)
-        self.final_attn_linear = nn.Linear(latent_dim, 1)
+        if final_norm_type is None:
+            self.final_norm = None
+        else:
+            self.final_norm = _get_norm(final_norm_type, latent_dim)
 
     @property
     def flatten_ts(self) -> bool:
@@ -252,17 +318,25 @@ class Transformer(ExtractorBase):
         return self.latent_dim
 
     def _aggregate(self, net: Tensor) -> Tensor:
-        a_hat = self.final_attn_linear(net)
-        a_prob = F.softmax(a_hat, dim=1)
-        return torch.sum(a_prob * net, dim=1)
+        if self.head_token is not None:
+            return net[:, -1]
+        return net.mean(1)
 
     def forward(self, net: Tensor) -> Tensor:
+        # input -> latent
         net = self.input_linear(net)
-        net = torch.tanh(net)
-        net = net * self.scaling
+        # concat head token
+        if self.head_token is not None:
+            expanded_token = self.head_token.expand(net.shape[0], 1, self.latent_dim)
+            net = torch.cat([net, expanded_token], dim=1)
+        # encode latent vector with transformer
         net = self.position_encoding(net)
         net = self.encoder(net, None)
-        return self._aggregate(net)
+        # aggregate & norm
+        net = self._aggregate(net)
+        if self.final_norm is not None:
+            net = self.final_norm(net)
+        return net
 
 
 __all__ = ["Transformer"]
